﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace AirplaneWithRadar
{
    internal class DrawningObjectAirplane : IDrawningObject
    {
        public DrawningAirplane _airplane { get; private set; }

        public DrawningObjectAirplane(DrawningAirplane airplane)
        {
            _airplane = airplane;
        }

        public float Step => _airplane?.Airplane?.Step ?? 0;

        public (float Left, float Right, float Top, float Bottom) GetCurrentPosition()
        {
            return _airplane?.GetCurrentPosition() ?? default;
        }

        public void MoveObject(Direction direction)
        {
            _airplane?.MoveTransport(direction);
        }

        public void SetObject(int x, int y, int width, int height)
        {
            _airplane.SetPosition(x, y, width, height);
        }

        void IDrawningObject.DrawningObject(Graphics g)
        {
            _airplane.DrawTransport(g);
        }
        public string GetInfo() => _airplane?.GetDataForSave() ?? string.Empty;
        public static IDrawningObject Create(string data) => new DrawningObjectAirplane(data.CreateDrawningAirplane());
        public bool Equals(IDrawningObject? other)
        {
            if (other is not DrawningObjectAirplane otherAirplane)
            {
                return false;
            }
            var entity = _airplane.Airplane;
            var otherEntity = otherAirplane._airplane.Airplane;
            if (entity.GetType() != otherEntity.GetType() ||
                entity.Speed != otherEntity.Speed ||
                entity.Weight != otherEntity.Weight ||
                entity.BodyColor != otherEntity.BodyColor)
            {
                return false;
            }
            if (entity is EntityAirplaneWithRadar entityAirplaneWithRadar &&
                otherEntity is EntityAirplaneWithRadar otherEntityAirplaneWithRadar && (
                entityAirplaneWithRadar.Ladder != otherEntityAirplaneWithRadar.Ladder ||
                entityAirplaneWithRadar.DopColor != otherEntityAirplaneWithRadar.DopColor ||
                entityAirplaneWithRadar.Window != otherEntityAirplaneWithRadar.Window ||
                entityAirplaneWithRadar.Radar != otherEntityAirplaneWithRadar.Radar))
            {
                return false;
            }
            return true;
        }
    }
}
