﻿namespace AirplaneWithRadar
{
	// Параметризованный набор объектов
	internal class SetAirplaneGeneric<T>
		where T : class, IEquatable<T>
    {
        private readonly List<T> _places;
        public int Count => _places.Count;
        private readonly int _maxCount;
        // Конструктор
        public SetAirplaneGeneric(int count)
		{
            _maxCount = count;
            _places = new List<T>();
        }
		// Добавление объекта в набор
		public int Insert(T airplane)
		{
            if (_places.Count > _maxCount)
            {
                return -1;
            }
            // вставка в начало набора
            return Insert(airplane, 0);
		}
		// Добавление объекта в набор на конкретную позицию
		public int Insert(T airplane, int position)
		{
            if (_places.Contains(airplane))
            {
                throw new ArgumentException($"Объект {airplane} уже присутствует в наборе");
            }
            if (position < 0 || position > Count || _maxCount == Count)
            {
                throw new StorageOverflowException(_maxCount);
            }
            _places.Insert(position, airplane);
            return position;
		}
		// Удаление объекта из набора с конкретной позиции
		public T Remove(int position)
		{
            if (position >= Count || position < 0)
            {
                throw new AirplaneNotFoundException(position);
            }
            T airplane = _places[position];
            _places.RemoveAt(position);
            return airplane;
                
        }
        // Получение объекта из набора по позиции
        public T this[int position]
        {
            get
            {
                if (position >= _places.Count || position < 0)
                {
                    return null;
                }
                return _places[position];
            }
            set
            {
                if (position >= _places.Count || position < 0)
                {
                    return;
                }
                Insert(value, position);

            }
        }
        public IEnumerable<T> GetAirplane()
        {
            foreach (var airplane in _places)
            {
                if (airplane != null)
                {
                    yield return airplane;
                }
                else
                {
                    yield break;
                }
            }
        }
        /// <summary>
        /// Сортировка набора объектов
        /// </summary>
        /// <param name="comparer"></param>
        public void SortSet(IComparer<T> comparer)
        {
            if (comparer == null)
            {
                return;
            }
            _places.Sort(comparer);
        }
    }
}